/*---------------------------------------------------------------------------*
  Project:  Horizon
  File:     demo_TrianglesRenderData.h

  Copyright (C)2009-2013 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  $Rev: 50863 $
 *---------------------------------------------------------------------------*/

#ifndef DEMO_TRIANGLES_RENDER_DATA_H_
#define DEMO_TRIANGLES_RENDER_DATA_H_

#include <nn/gx.h>
#include <nn/math.h>

#include "demo/Utility/demo_MemoryManager.h"

#include "demo/RenderData/demo_RenderData.h"

namespace demo
{
    /* Please see man pages for details 
    

    
    */
    enum VertexAttributes
    {
        VERTEX_NONE_ATTRIBUTE = 0,            //
        VERTEX_POSITION_ATTRIBUTE = (1 << 0), //
        VERTEX_COLOR_ATTRIBUTE    = (1 << 1), //
        VERTEX_TEXCOORD_ATTRIBUTE = (1 << 2), //
        VERTEX_NORMAL_ATTRIBUTE   = (1 << 3), //
        VERTEX_TANGENT_ATTRIBUTE  = (1 << 4)  //
    };

    /* Please see man pages for details 
    

    
    */
    const int VERTEX_POSITION_ATTRIBUTE_SIZE = 4;
    
    /* Please see man pages for details 
    

    
    */
    const int VERTEX_COLOR_ATTRIBUTE_SIZE  = 4;    
    
    /* Please see man pages for details 
    

    
    */       
    const int VERTEX_TEXCOORD_ATTRIBUTE_SIZE = 3;
    
    /* Please see man pages for details 
    

    
    */       
    const int VERTEX_NORMAL_ATTRIBUTE_SIZE = 3;
    
    /* Please see man pages for details 
    

    
    */  
    const int VERTEX_TANGENT_ATTRIBUTE_SIZE = 3;

    /* Please see man pages for details 
    

    
    */  
    const u32 MAX_TRIANGLES_NUM = 8192;

    /* Please see man pages for details 
    

    
    */  
    const u32 MAX_SQUARES_NUM = MAX_TRIANGLES_NUM / 2;

    /* Please see man pages for details 
    

    
    */
    const u32 MAX_TRIANGLE_VERTICES_NUM = MAX_TRIANGLES_NUM * 3;
    
    /* Please see man pages for details 
    

    
    */
    const u32 MAX_INDICES_NUM = MAX_TRIANGLES_NUM * 3;
    
    /* Please see man pages for details 
    

    
    */
    const GLenum ARRAY_BUFFER_DATA_TYPE = GL_ARRAY_BUFFER;

    /* Please see man pages for details 
    

    
    */
    const GLenum ELEMENT_ARRAY_BUFFER_DATA_TYPE = GL_ELEMENT_ARRAY_BUFFER;

    /* Please see man pages for details 
    

    
    */
    class TrianglesRenderData : public RenderData
    {
    public:
        /* Please see man pages for details 
        

        
        */
        TrianglesRenderData(void);

        /* Please see man pages for details 
        

        
        */
        virtual ~TrianglesRenderData(void);

    public:
        /* Please see man pages for details 
        

        
        */
        virtual void Initialize(void);

        /* Please see man pages for details 
        

        
        */
        virtual void Finalize(void);

    public:
        /* Please see man pages for details 
        

        
        
        
        
        
        
        */
        void InitializeVertexBuffers(const u32 vertexAttributes,
            const GLenum triangleType, const u32 verticesNum, const u32 trianglesNum);

    protected:        
        void SetVertexAttributes(const u32 vertexAttributes);        
        void SetTriangleType(const GLenum triangleType);
        void SetVerticesNum(const u32 verticesNum);
        void SetTrianglesNum(const u32 trianglesNum);
        void CreateArrays(void);
        void DestroyArrays(void);

    public:
        /* Please see man pages for details 
        

        
        
        
        */
        void SetPackedVerticesNum(const u32 packedVerticesNum);
        
        /* Please see man pages for details 
        

        
        
        
        */
        void AddPackedVerticesNum(const u32 packedVerticesNum);

        /* Please see man pages for details 
        

        
        
        
        */
        u32 GetPackedVerticesNum(void) const;
    
    public:
        /* Please see man pages for details 
        

        
        
        
        */
        void SetPackedTrianglesNum(const u32 packedTrianglesNum);

        /* Please see man pages for details 
        

        
        
        
        */
        void AddPackedTrianglesNum(const u32 packedTrianglesNum);

        /* Please see man pages for details 
        

        
        
        
        */
        u32 GetPackedTrianglesNum(void) const;

    public:
        /* Please see man pages for details 
        

        
        
        */
        void DrawPackedTriangles(void);

    public:
        /* Please see man pages for details 
        

        
        */
        void ClearPackedNum(void);

        /* Please see man pages for details 
        

        
        */
        void SetPackedArraysSize(void);

    public:
        /* Please see man pages for details 
        

        
        */
        void UpdateBuffers(void);

        /* Please see man pages for details 
        

        
        */
        void SetUpdateBufferBeforeDraw(const bool updateBufferFlag);
    protected:
        void DestroyBuffers(void);    

    public:
        /* Please see man pages for details 
        

        
                                
        
        
        
        
        */
        void SetPosition(const u32 index,
            const GLfloat positionX, const GLfloat positionY, const GLfloat positionZ);
    
        /* Please see man pages for details 
        

        
                               
        
        
        
        
        
        */
        void SetPosition(const u32 index,
            const GLfloat positionX, const GLfloat positionY, 
            const GLfloat positionZ, const GLfloat positionW);

        /* Please see man pages for details 
        

        
                                
        
        
        
        
        */
        void SetColor(const u32 index, 
            const GLfloat red, const GLfloat green, const GLfloat blue);

        /* Please see man pages for details 
        

        
                                
        
        
        
        
        
        */
        void SetColor(const u32 index,
            const GLfloat red, const GLfloat green, const GLfloat blue, const GLfloat alpha);

        /* Please see man pages for details 
        

        
                                        
        
        
        
        */
        virtual void SetColor(const GLfloat red, const GLfloat green, const GLfloat blue);

        /* Please see man pages for details 
        

        
                                
        
        
        
        
        
        */
        virtual void SetColor(const GLfloat red, const GLfloat green, const GLfloat blue, const GLfloat alpha);

        /* Please see man pages for details 
        

        
                                
        
        
        
        */
        void SetTexcoord(const u32 index,
            const GLfloat texcoordS, const GLfloat texcoordT);

        /* Please see man pages for details 
        

        
                                
        
        
        
        
        */
        void SetTexcoord(const u32 index,
            const GLfloat texcoordS, const GLfloat texcoordT, const GLfloat texcoordR);

        /* Please see man pages for details 
        

        
                                
        
        
        
        
        */
        void SetNormal(const u32 index,
            const GLfloat normalX, const GLfloat normalY, const GLfloat normalZ);
        
        /* Please see man pages for details 
        

        
                                
        
        
        
        
        */
        void SetTangent(const u32 index,
            const GLfloat tangentX, const GLfloat tangentY, const GLfloat tangentZ);

        /* Please see man pages for details 
        

        
                               
        
        
        
        
        */
        void SetIndex(const u32 index,
            const GLuint index0, const GLuint index1, const GLuint index2);

        /* Please see man pages for details 
        

        
                        
        
        
        
        
        
        */
        void SetSquareIndex(const u32 square_index,
            const GLuint index0, const GLuint index1,
            const GLuint index2, const GLuint index3);
    
    public:
        /* Please see man pages for details 
        

        
        
        
        
        
        */
        void SetWorldPosition(const f32 worldPositionX, const f32 worldPositionY, const f32 worldPositionZ);

        /* Please see man pages for details 
        

        
        
        
        
        
        */
        void TranslateWorldPosition(const f32 worldPositionX, const f32 worldPositionY, const f32 worldPositionZ);

        /* Please see man pages for details 
        

        
        
        
        
        
        */
        void GetWorldPosition(f32& worldPositionX, f32& worldPositionY, f32& worldPositionZ);

        /* Please see man pages for details 
        

        
        
        
        
        
        */
        void SetWorldAngle(const f32 worldAngleX, const f32 worldAngleY, const f32 worldAngleZ);
        
        /* Please see man pages for details 
        

        
        
        
        
        
        */
        void RotateWorldAngle(const f32 worldAngleX, const f32 worldAngleY, const f32 worldAngleZ);

        /* Please see man pages for details 
        

        
        
        
        
        
        */
        void GetWorldAngle(f32& worldAngleX, f32& worldAngleY, f32& worldAngleZ);
        
    public:
        /* Please see man pages for details 
        

        
        
        
        */
        virtual nn::math::MTX44 GetWorldMatrix(void) const;

    public:
        /* Please see man pages for details 
        

        
        */
        virtual void Draw(void);

    protected:
        virtual void DrawElements(void);

    public:
        /* Please see man pages for details 
        

        
        */
        virtual void EnableVertexAttributes(void);

        /* Please see man pages for details 
        

        

        
        */
        void SetEnableVertexAttributesBeforeDraw(const bool updateAttributesFlag);

    protected:
        u32 m_ShaderType;
        u32 m_VertexAttributes;
        GLenum m_TriangleType;
        u32 m_VerticesNum;
        u32 m_TrianglesNum;
        u32 m_PackedVerticesNum;
        u32 m_PackedTrianglesNum;

    protected:
        u32 m_PositionArraySize;
        u32 m_ColorArraySize;
        u32 m_TexcoordArraySize;
        u32 m_NormalArraySize;        
        u32 m_TangentArraySize;
        u32 m_TotalArraySize;
        u32 m_OldTotalArraySize;

        u32 m_IndexArraySize;
        u32 m_OldIndexArraySize;
    
    protected:
        GLfloat* m_PositionArray;
        GLfloat* m_ColorArray;
        GLfloat* m_TexcoordArray;
        GLfloat* m_NormalArray;        
        GLfloat* m_TangentArray;        
        GLushort* m_IndexArray;
    
    protected:
        GLuint m_ArrayBufferId;
        GLuint m_ElementArrayBufferId;
   
    protected:
        bool m_InitArrayFlag;
        bool m_InitBufferFlag;
        bool m_UpdateBufferFlag;
        bool m_UpdateAttributesFlag;

    protected:
        f32 m_PositionX;
        f32 m_PositionY;
        f32 m_PositionZ;
        f32 m_AngleX;
        f32 m_AngleY;
        f32 m_AngleZ;
    };

}

#endif
