// ----------------------------------------------------------------------
// Main menu class
// ----------------------------------------------------------------------

// ----------------------------------------------------------------------
// include system header
// ----------------------------------------------------------------------
#include	<nn.h>
#include	<nn/gx.h>
#include	<nn/math.h>
#include	<nn/os.h>
#include	<nn/hid.h>
#include	<nn/fnd.h>
#include	<nn/fs.h>

#include	<string.h>

// ----------------------------------------------------------------------
// include original header
// ----------------------------------------------------------------------
#include	"CTR_Define.h"
#include	"CtrlGraphics.h"
#include	"CtrlCamera.h"
#include	"TxtWindow.h"
#include	"TxtWinMenu.h"

// ----------------------------------------------------------------------
// using namespace
// ----------------------------------------------------------------------
using namespace nn::hid;

typedef struct
{
	int	mgmode;
	int frmparam;
	int spdparam;
	int sknparam;
}MGRlibParam, *LPMGRLibParam;

// ----------------------------------------------------------------------
// global
// ----------------------------------------------------------------------
static int g_quality	= 2;
static MGRlibParam  g_MGRLibParam;

// ----------------------------------------------------------------------
// SetDrawQuality
// ----------------------------------------------------------------------
// Content
//   Changes the render quality.
//   1: Render as is
//   2: Make height and width to 1/2, enlarge then render
//   3: Make height and width to 1/4, enlarge then render
//   Method 1 is higher quality, and method 3 is faster.
// ----------------------------------------------------------------------
// Arguments
//   n		[ I ] Specifies parameters after modification.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void SetDrawQuality(int n)
{
	switch(n)
	{
	case 1:g_quality = 1;break;
	case 2:g_quality = 2;break;
	case 3:g_quality = 4;break;
	}
}

// ----------------------------------------------------------------------
// CTxtWinMenu::GetDrawQuality
// ----------------------------------------------------------------------
// Content
//   Gets the render quality.
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   Returns the quality value (1, 2, 4).
// ----------------------------------------------------------------------
int CTxtWinMenu::GetDrawQuality()
{
	return g_quality;
}

// ----------------------------------------------------------------------
// SetMgMode/SetFrmParam/SetSpdParam/SetSknParam
// ----------------------------------------------------------------------
// Content
//   Changes the MGRLib parameters.
// ----------------------------------------------------------------------
// Arguments
//   n		[ I ] Specifies parameters after modification.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
// Version 1.0.1 Added SetSknParam
// ----------------------------------------------------------------------
void SetMgMode  (int n) { g_MGRLibParam.mgmode   = n; }
void SetFrmParam(int n) { g_MGRLibParam.frmparam = n; }
void SetSpdParam(int n) { g_MGRLibParam.spdparam = n; }
void SetSknParam(int n) { g_MGRLibParam.sknparam = n; }

// ----------------------------------------------------------------------
// CTxtWinMenu::GetMgMode/CTxtWinMenu::GetFrmparam/CTxtWinMenu::GetSpdparam
// CTxtWinMenu::GetSknparam
// ----------------------------------------------------------------------
// Content
//   Gets the MGRLib parameter value.
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   Returns each value.
// ----------------------------------------------------------------------
// Version 1.0.1 Added GetSknparam
// ----------------------------------------------------------------------
int CTxtWinMenu::GetMgMode()	{ return g_MGRLibParam.mgmode;	 }
int CTxtWinMenu::GetFrmparam()	{ return g_MGRLibParam.frmparam; }
int CTxtWinMenu::GetSpdparam()	{ return g_MGRLibParam.spdparam; }
int CTxtWinMenu::GetSknparam()	{ return g_MGRLibParam.sknparam; }

// ----------------------------------------------------------------------
// Content
//   CTxtWinMenu::CTxtWinMenu
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
CTxtWinMenu::CTxtWinMenu()
	: CTxtWindow("Menu"),
	m_menu(3, "PROC SIZE"),
	m_menu_sa(3, "SETTING APPLICATION"),
	m_menu_sm(5, "SETTING MGRLIB"),
	m_menu_sc(9, "SETTING GESTURE")
{
	// ----------------------------------------------------------------------
	// Application Menu
	// ----------------------------------------------------------------------
	m_menu_sa.SetItemOnOff	(0, eMENU_T_SA_ENABLE,				"Motion Gesture     ", true, NULL);
	m_menu_sa.SetItemMinMax	(1, eMENU_T_SA_DRAWQUALITY,			"Draw Quality       ",  1,  3, 2, SetDrawQuality);
	m_menu_sa.SetItemExit	(2, eMENU_T_SA_EXIT,				"Exit               ", NULL);

	// ----------------------------------------------------------------------
	// MGRLib Menu
	// ----------------------------------------------------------------------
	m_menu_sm.SetItemMinMax	(0, eMENU_T_SM_MGMODE,				"MG Mode            ",  0,  1, 0, SetMgMode);
	m_menu_sm.SetItemMinMax	(1, eMENU_T_SM_FRMPARAM,			"Frame Param        ",  0, 10, 5, SetFrmParam);
	m_menu_sm.SetItemMinMax	(2, eMENU_T_SM_SPDPARAM,			"Speed Param        ",  0,  4, 2, SetSpdParam);
	m_menu_sm.SetItemMinMax	(3, eMENU_T_SM_SKNPARAM,			"Skin  Param        ",  0,200, 100, SetSknParam, 10);
	m_menu_sm.SetItemExit	(4, eMENU_T_SM_EXIT,				"Exit               ", NULL);

	// ----------------------------------------------------------------------
	// Camera menu
	// ----------------------------------------------------------------------
	m_menu_sc.SetItemMinMax	(0, eMENU_T_SC_EXPOSURE,			"Exposure           ", -5,  5, 0, SetExporse);
	m_menu_sc.SetItemMinMax	(1, eMENU_T_SC_WHITEBALANCE,		"White Balance      ",  0,  5, 2, SetWhiteBalance);
	m_menu_sc.SetItemMinMax	(2, eMENU_T_SC_SHARPNESS,			"Sharpness          ", -4,  5, 1, SetSharpness);
	m_menu_sc.SetItemOnOff	(3, eMENU_T_SC_AUTOEXPOSURE,		"Auto Exposure      ", false,	  SetAutoExporse);
	m_menu_sc.SetItemOnOff	(4, eMENU_T_SC_AUTOWHITEBALANCE,	"Auto White Balance ", false,	  SetAutoWhiteBalance);
	m_menu_sc.SetItemMinMax	(5, eMENU_T_SC_PHOTOMODE,			"Photo Mode         ",  0,  4, 1, SetPhotoMode);
	m_menu_sc.SetItemMinMax	(6, eMENU_T_SC_CONTRAST,			"Contrast           ",  0, 10, 6, SetContrast);
	m_menu_sc.SetItemMinMax	(7, eMENU_T_SC_LENSCORRECTION,		"Lenscorrection     ",  0,  2, 2, SetLensCorrection);
	m_menu_sc.SetItemExit	(8, eMENU_T_SC_EXIT,				"Exit               ", NULL);

	m_menu_sc.SetItemStrFunc(eMENU_T_SC_PHOTOMODE,		GetPhotoModeStr);
	m_menu_sc.SetItemStrFunc(eMENU_T_SC_WHITEBALANCE,	GetWhiteBalanceStr);
	m_menu_sc.SetItemStrFunc(eMENU_T_SC_CONTRAST,		GetContrastStr);
	m_menu_sc.SetItemStrFunc(eMENU_T_SC_LENSCORRECTION,	GetLensCorrectionStr);

	// ----------------------------------------------------------------------
	// Root Menu
	// ----------------------------------------------------------------------
	m_menu.SetSubMenu(0, eMENU_T_SETTING_APPLICATION,	"Setting Application ", &m_menu_sa, NULL);
	m_menu.SetSubMenu(1, eMENU_T_SETTING_MGRLIB,		"Setting MGRLib      ", &m_menu_sm, NULL);
	m_menu.SetSubMenu(2, eMENU_T_SETTING_CAMERA,		"Setting Camera      ", &m_menu_sc, NULL);

	g_MGRLibParam.mgmode   = 0;
	g_MGRLibParam.frmparam = 5;
	g_MGRLibParam.spdparam = 2;
	g_MGRLibParam.sknparam = 100;
}

// ----------------------------------------------------------------------
// CTxtWinMenu::~CTxtWinMenu
// ----------------------------------------------------------------------
// Content
//   Destructor
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
CTxtWinMenu::~CTxtWinMenu()
{
}

// ----------------------------------------------------------------------
// CTxtWinMenu::Draw
// ----------------------------------------------------------------------
// Content
//   Displays Window.
// ----------------------------------------------------------------------
// Arguments
//   graphics		[ I ] Specify graphics class.
//   display		[ I ] Specify Window show display.
//   sx				[ I ] Specify Window display x-coordinate.
//   sy				[ I ] Specify Window display y-coordinate.
// ----------------------------------------------------------------------
// Return value
//   Always returns 0.
// ----------------------------------------------------------------------
int CTxtWinMenu::Draw(CCtrlGraphics* graphics, const s32 display, int sx, int sy)
{
	char	str[64];
	int		n		 = 0;
	int		menu_num = m_menu.GetCount();

	CTxtWindow::SetLines(menu_num);
	for (n = 0 ; n < menu_num ; n++)
	{
		m_menu.GetItemText(n, str);
		CTxtWindow::SetText(n, str, 0.0, 0.0, 0.0);
	}

	CTxtWindow::DrawWindow(graphics, display, sx, sy);

	return 0;
}

// ----------------------------------------------------------------------
// CTxtWinMenu::IsEnableHandGesture
// ----------------------------------------------------------------------
// Content
//   Gets whether to perform motion gesture recognition.
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   Returns true when on; returns false when off.
// ----------------------------------------------------------------------
bool CTxtWinMenu::IsEnableHandGesture()
{
	return m_menu_sa.GetItemOnOff(eMENU_T_SA_ENABLE);
}

// ----------------------------------------------------------------------
// CTxtWinMenu::Ctrl
// ----------------------------------------------------------------------
// Content
//   Performs operations with PAD.
//   - UP button: moves to item above.
//   - DOWN button: moves to item below.
//   - A Button: Confirm or adds to the value.
//   - B Button: Confirm or subtracts from the value.
// ----------------------------------------------------------------------
// Arguments
//   padStatus			[ I ] Specifies pad status.
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CTxtWinMenu::Ctrl(PadStatus* padStatus)
{
	// ----------------------------------------------------------------------
	// Up Button
	// ----------------------------------------------------------------------
	if (padStatus->trigger & BUTTON_UP)
	{
		m_menu.SetPrev();
	}

	// ----------------------------------------------------------------------
	// Down Button
	// ----------------------------------------------------------------------
	if (padStatus->trigger & BUTTON_DOWN)
	{
		m_menu.SetNext();
	}

	// ----------------------------------------------------------------------
	// A Button Determination (plus)
	// ----------------------------------------------------------------------
	if (padStatus->trigger & BUTTON_A)
	{
		m_menu.SetSelect(true);
	}

	// ----------------------------------------------------------------------
	// B Button Determination (minus)
	// ----------------------------------------------------------------------
	if (padStatus->trigger & BUTTON_B)
	{
		m_menu.SetSelect(false);
	}
}
