//**************************************************************************************
//
//		Motion Gesture Recognition Library (MGRLib)
//
//		MGRLib Native Source File
//
//		Copyright   :: (C) PANASONIC CORPORATION, SYSTEM NETWORKS COMPANY 2010
//		Date        :: 2010.03.23
//		CPU         :: **
//		Language    :: C,C++
//		File Name   :: demo.cpp
//
//**************************************************************************************

// ----------------------------------------------------------------------
// define
// ----------------------------------------------------------------------
#define		CPLUSPLUS

// ----------------------------------------------------------------------
// include system header
// ----------------------------------------------------------------------
#include	<nn.h>
#include	<nn/os.h>
#include	<nn/fs.h>
#include	<nn/fnd.h>
#include	<nn/hid.h>
#include	<nn/math.h>
#include	<nn/hid.h>
#include	<nn/camera.h>
#include	<nn/y2r.h>
#include	<nn/hid.h>
#include	<nn/applet.h>

// ----------------------------------------------------------------------
// include mgr header
// ----------------------------------------------------------------------
#include	"mgr.h"					// mgr header
using namespace mw::mgr;

// ----------------------------------------------------------------------
// include original header
// ----------------------------------------------------------------------
#include	"CTR_Define.h"
#include	"CtrlGraphics.h"
#include	"CtrlCamera.h"
#include	"TxtMenu.h"
#include	"TxtWindow.h"
#include	"TxtWinStartup.h"
#include	"TxtWinMenu.h"
#include	"TxtWinResultMgr.h"
#include	"TxtWinDebugTimer.h"
#include	"DebugWriterMgr.h"
#include	"appdefine.h"

// ----------------------------------------------------------------------
// using namespace
// ----------------------------------------------------------------------
using namespace nn::hid;
using namespace nn::os;
using namespace nn::math;

// ----------------------------------------------------------------------
// global
// ----------------------------------------------------------------------
extern nn::fnd::ExpHeap gAppHeap;
static u8 *rgbBuffer;

// ----------------------------------------------------------------------
// nnMain
// ----------------------------------------------------------------------
void nnMain(void)
{
	unsigned char	*extBuf			= NULL;
	u32				workSize        = 0;
	MGConfig		mgConfig;

	PSCimg			mgCimg;

	MGResult		mgResult;
	int				bodynum			= 0;
	MGObject		*object			= NULL;

	int 			n				= 0;
	int 			x				= 0;
	int 			y				= 0;
	int				i				= 0;
	int				ret				= 0;

	int				proc_width		= 320;				// Overall process width: image input through rendering (excluding recognition)
	int				proc_height		= 240;				// Overall process height: image input through rendering (excluding recognition)
	Size			proc_size		= SIZE_QVGA;
	FrameRate		proc_rate		= FRAME_RATE_30;
	int				proc_per		= 2;				// Input process image / recognition process image

	int				view_width		= nn::gx::CTR::DISPLAY1_HEIGHT;		// Width of View to display the camera image   (DISPLAY1_HEIGHT = 320) 
	int				view_height		= nn::gx::CTR::DISPLAY1_WIDTH;		// Height of View to display the camera image   (DISPLAY1_WIDTH = 240)

	int				mgr_width		= 160;				// Recognition processing image width
	int				mgr_height		= 120;				// Recognition processing image height

	int				txtWinSort		= 0;

	CCtrlGraphics	graphics;					// Graphics control class
	CCtrlCamera		camera;						// Camera control class

	// Window class
	CTxtWinStartup			startWnd;			// Start up Window
	CTxtWinMenu				menuWnd;			// Menu Window
	CTxtWinResultMgr		resultWnd;			// Result Window
	CTxtWinDebugTimer		dbgTimerWnd(8);		// Debug timer display window

	CDebugWriterMgr	dbgWriter;			// Debugging output.
	bool			dbg_init		= false;	

	char str[64];

	// ======================================================================

	PRINT_OUT("Demo Start\n");

    nn::Result result;

    // fs initialization
    nn::fs::Initialize();

    // Initialize hid
    result = nn::hid::Initialize();
    NN_UTIL_PANIC_IF_FAILED(result);

    // Call only the nn::applet::Enable function to also allow execution from the HOME Menu
    // HOME Menu transitions, POWER Button presses, and sleep are all unsupported
    nn::applet::Enable();
    
	// Prepare pad
	PadReader		padReader;
	PadStatus		padStatus;

	OS_MEM_INIT();
	FILE_IO_INIT();
	OS_TICK_INIT();

 	// ----------------------------------------------------------------------
	// Initialize graphics class
	// ----------------------------------------------------------------------
	graphics.InitializeGraphics();

 	// ----------------------------------------------------------------------
	// Display Start up Window, Support QQVGA/DS_LCD only
	// ----------------------------------------------------------------------
	startWnd.DoModal(&graphics, NN_GX_DISPLAY1, -1, -1);
	
	eSTART_MODE eStartMode = startWnd.GetStartMode();

	if (eStartMode == eSTART_VGA)
	{
		proc_width		= 640;				// Input image/Render image width
		proc_height		= 480;				// Input image/Render image height
		proc_size		= SIZE_VGA;			// Input size
		proc_rate		= FRAME_RATE_20;	// Input frame rate
		mgr_width		= 160;				// Recognition processing image width
		mgr_height		= 120;				// Recognition processing image height
		proc_per		= 4;				// Make height and width each 1/4
	}
	else if (eStartMode == eSTART_QVGA)
	{
		proc_width		= 320;				// Input image/Render image width
		proc_height		= 240;				// Input image/Render image height
		proc_size		= SIZE_QVGA;		// Input size
		proc_rate		= FRAME_RATE_30;	// Input frame rate
		mgr_width		= 160;				// Recognition processing image width
		mgr_height		= 120;				// Recognition processing image height
		proc_per		= 2;				// Make height and width each 1/2
	}
	else if (eStartMode == eSTART_QQVGA)
	{
		proc_width		= 160;				// Input image/Render image width
		proc_height		= 120;				// Input image/Render image height
		proc_size		= SIZE_QQVGA;		// Input size
		proc_rate		= FRAME_RATE_30;	// Input frame rate
		mgr_width		= 160;				// Recognition processing image width
		mgr_height		= 120;				// Recognition processing image height
		proc_per		= 1;				// Make height and width each 1/1
	}
	else if (eStartMode == eSTART_DS_LCD)
	{
		proc_width		= 256;				// Input image/Render image width
		proc_height		= 192;				// Input image/Render image height
		proc_size		= SIZE_DS_LCD;		// Input size
		proc_rate		= FRAME_RATE_30;	// Input frame rate
		mgr_width		= 128;				// Recognition processing image width
		mgr_height		= 96;				// Recognition processing image height
		proc_per		= 2;				// Make height and width each 1/2
	}

 	// ----------------------------------------------------------------------
	// Initialize camera class
 	// ----------------------------------------------------------------------
	camera.InitializeCamera(SELECT_IN1, proc_size, proc_rate);

	dbgTimerWnd.SetTimerText(0, "recv camera    ");
	dbgTimerWnd.SetTimerText(1, "create cimg    ");
	dbgTimerWnd.SetTimerText(2, "proc exec      ");
	dbgTimerWnd.SetTimerText(3, "draw disp0     ");
	dbgTimerWnd.SetTimerText(4, "create texture ");
	dbgTimerWnd.SetTimerText(5, "draw disp1     ");
	dbgTimerWnd.SetTimerText(6, "other          ");
	dbgTimerWnd.SetTimerText(7, "swap buffer    ");

	// ----------------------------------------------------------------------
	// Settings
	// ----------------------------------------------------------------------
	mgConfig.xsize		= mgr_width;				// Image data width		(128, 160)
	mgConfig.ysize		= mgr_height;				// Image data height		(96, 120)
	mgConfig.mgrect.sx	= 0;						// Rectangular region targeted for processing
	mgConfig.mgrect.sy	= 0;						// Rectangular region targeted for processing
	mgConfig.mgrect.ex	= mgr_width - 1;			// Rectangular region targeted for processing
	mgConfig.mgrect.ey	= mgr_height - 1;			// Rectangular region targeted for processing
	mgConfig.mgmode		= 0;						// Recognition movement mode		(0, 1)
	mgConfig.frmparam	= 5;						// Difference sensitivity between frames	(0 ～ 10)
	mgConfig.spdparam	= 2;						// Speed sensitivity				(0 ～ 4)
	mgConfig.sknparam	= 100;						// Flesh-color filter parameter (0 through 200)		Note: Added from version 1.0.1
	mgConfig.reserve0	= 0;						// Reserve Flag 0
	mgConfig.reserve1	= 0;						// Reserve Flag 1							Note: Added from version 1.0.1
	mgConfig.reserve2	= 0;						// Reserve Flag 2							Note: Added from version 1.0.1
	mgConfig.reserve3	= 0;						// Reserve Flag 3							Note: Added from version 1.0.1

	// ----------------------------------------------------------------------
	// Library function: Get size of required working memory
	// ----------------------------------------------------------------------
	ret = MGRcgGetWorkSize(&mgConfig, &workSize);
	if (ret != PSERR_TRUE)
	{
		PRINT_OUT("MGRcgGetWorkSize NG\n");
		return;
	}
	PRINT_OUT("MGRcgGetWorkSize OK[%ld]\n", workSize);

	extBuf = (u8*)OS_MALLOC(workSize);
	if (extBuf == NULL)
	{
		PRINT_OUT("malloc extBuf NG\n");
		return;
	}

    rgbBuffer = (u8*)OS_MALLOC_EX( 640 * 480 * 4, 64 );

	mgCimg.xSize = mgr_width;
	mgCimg.ySize = mgr_height;
	mgCimg.pcImg = (RGBColor*)OS_MALLOC(mgr_width * mgr_height * sizeof(RGBColor));
	if (mgCimg.pcImg == NULL)
	{
		PRINT_OUT("malloc pcImg NG\n");
		OS_FREE(extBuf);
		return ;
	}

	// ----------------------------------------------------------------------
	// Library function: Initialization
	// ----------------------------------------------------------------------
	ret = MGRcgProcessInit(&mgConfig, extBuf);
	if (ret != PSERR_TRUE)
	{
		PRINT_OUT("MGRcgProcessInit NG\n");
		OS_FREE(extBuf);
		OS_FREE(mgCimg.pcImg);
		return;
	}
	PRINT_OUT("MGRcgProcessInit OK\n");

	// ----------------------------------------------------------------------
	// Settings for each Window
	// ----------------------------------------------------------------------
	menuWnd.SetWindowWidth(nn::gx::DISPLAY0_HEIGHT);

	resultWnd.SetWindowWidth(306);
	resultWnd.SetWindowHeight(120);

	menuWnd.SetForcus(true);

	// ----------------------------------------------------------------------
	// Main loop.
	// ----------------------------------------------------------------------
	int frame	 = 0;
	while(1)
	{
		dbgTimerWnd.TimerAllStart();

		// ----------------------------------------------------------------------
		// Get camera data
		// ----------------------------------------------------------------------
		dbgTimerWnd.TimerStart(0);
		camera.Receive(rgbBuffer);
		dbgTimerWnd.TimerEnd(0);

		// ----------------------------------------------------------------------
		// Create PSCimg (since the top is (0, 0), reverse top and bottom) 
		// ----------------------------------------------------------------------
		dbgTimerWnd.TimerStart(1);
		if (menuWnd.IsEnableHandGesture() == true)
		{
			for (y = 0 ; y < proc_height ; y+=proc_per)
			{
				u32* dest = (u32*)((u8*)(mgCimg.pcImg) + (proc_height - 1 - y) / proc_per * mgr_width * 4);
				u32* src  = (u32*)(rgbBuffer + y * proc_width * 4);
				for (x = 0 ; x < proc_width ; x+=proc_per)
				{
					*dest++  = *src++;
					src		+= proc_per - 1;
				}
			}
		}
		dbgTimerWnd.TimerEnd(1);

		// ----------------------------------------------------------------------
		// Recognition
		// ----------------------------------------------------------------------
		mgResult.bodynum	= 0;
		for (i = 0 ; i < MGRCG_BDYMAX ; i++)
		{
			mgResult.mgobject[i].id = 0;
			mgResult.mgobject[i].rect.sx = 0;
			mgResult.mgobject[i].rect.sy = 0;
			mgResult.mgobject[i].rect.ex = 0;
			mgResult.mgobject[i].rect.ey = 0;
			mgResult.mgobject[i].center.x = 0;
			mgResult.mgobject[i].center.y = 0;
		}
		mgResult.reserve0	= 0;
		mgResult.reserve1	= 0;
		mgResult.reserve2	= 0;
		mgResult.reserve3	= 0;

		mgConfig.mgmode		= menuWnd.GetMgMode();
		mgConfig.frmparam	= menuWnd.GetFrmparam();
		mgConfig.spdparam	= menuWnd.GetSpdparam();
		mgConfig.sknparam	= menuWnd.GetSknparam();	// Added from version 1.0.1
		
		dbgTimerWnd.TimerStart(2);
		if (menuWnd.IsEnableHandGesture() == true)
		{
			// ----------------------------------------------------------------------
			// Library function: execute recognition
			// ----------------------------------------------------------------------
			ret = MGRcgProcessExec(&mgConfig, &mgCimg, &mgResult);
		}
		dbgTimerWnd.TimerEnd(2);

		bodynum		= mgResult.bodynum;
		object		= mgResult.mgobject;

		// ======================================================================
		// Upper screen.
		// ======================================================================
		dbgTimerWnd.TimerStart(3);
		graphics.ClearDisplay(NN_GX_DISPLAY0);

		// ----------------------------------------------------------------------
		// Set the recognition result
		// ----------------------------------------------------------------------
		resultWnd.UpdateResult(&mgResult);

		// ----------------------------------------------------------------------
		// Render Window
		// ----------------------------------------------------------------------
		for (n = txtWinSort ; n < txtWinSort + 3 ; n++)
		{
			switch((n + 1) % 3)
			{
			case 0: menuWnd.Draw	(&graphics, NN_GX_DISPLAY0,   0,  0);	break;
			case 1: resultWnd.Draw	(&graphics, NN_GX_DISPLAY0,  40, 80);	break;
			case 2:	dbgTimerWnd.Draw(&graphics, NN_GX_DISPLAY0,  10, 60);	break;
			}
		}

		// ----------------------------------------------------------------------
		// Render frame in bottom right
		// ----------------------------------------------------------------------
		sprintf(str, "frame:[%08d]\n", frame++);
		graphics.SetColor(NN_GX_DISPLAY0, 0.0, 1.0, 0.0);
		graphics.DrawTextEx(NN_GX_DISPLAY0, 272, 214, str);

		// ----------------------------------------------------------------------
		// Swap
		// ----------------------------------------------------------------------
		graphics.SwapBuffer(NN_GX_DISPLAY0);

		dbgTimerWnd.TimerEnd(3);

		// ======================================================================
		// Bottom display
		// ======================================================================

		// ----------------------------------------------------------------------
		// Generate render texture
		// ----------------------------------------------------------------------
		u32 bmpWidth		= proc_width;
		u32 bmpHeight		= proc_height;
		u32 textureWidth	= 0;
		u32 textureHeight	= 0;

		dbgTimerWnd.TimerStart(4);
		int quality = menuWnd.GetDrawQuality();
		u8* texBuffer = graphics.GetTextureDataFromBmpFileData(bmpWidth, bmpHeight, rgbBuffer, textureWidth, textureHeight, quality);
		dbgTimerWnd.TimerEnd(4);

		// ----------------------------------------------------------------------
		// Rendering
		// ----------------------------------------------------------------------
		dbgTimerWnd.TimerStart(5);
		graphics.ClearDisplay(NN_GX_DISPLAY1);
		graphics.DrawCamera(texBuffer, textureWidth, textureHeight, bmpWidth, bmpHeight, quality);
		graphics.DrawDateTime();
		dbgTimerWnd.TimerEnd(5);

		// ----------------------------------------------------------------------
		// Render recognition results
		// ----------------------------------------------------------------------
		// Enclose recognition places in red frame
		// ----------------------------------------------------------------------
		dbgTimerWnd.TimerStart(6);
		object	= mgResult.mgobject;
		for (n = 0 ; n < bodynum ; n++ )
		{
			// Start point
			float sx	= (float)object->rect.sx * ((float)view_width  / (float)mgr_width);
			float sy	= (float)object->rect.sy * ((float)view_height / (float)mgr_height);

			// End point
			float ex	= (float)object->rect.ex * ((float)view_width  / (float)mgr_width);
			float ey	= (float)object->rect.ey * ((float)view_height / (float)mgr_height);

			graphics.SetColor(NN_GX_DISPLAY1, 1.0, 0.0, 0.0);
			graphics.DrawLine(NN_GX_DISPLAY1, sx, sy, ex, sy);
			graphics.DrawLine(NN_GX_DISPLAY1, ex, sy, ex, ey);
			graphics.DrawLine(NN_GX_DISPLAY1, ex, ey, sx, ey);
			graphics.DrawLine(NN_GX_DISPLAY1, sx, ey, sx, sy);
			object++;
		}

		// ----------------------------------------------------------------------
		// Menu operation through controller operation
		// ----------------------------------------------------------------------
		// Select Window with LR Buttons
		// Operate items with UP/DOWN Buttons
		// Modify values with AB Buttons
		// Output to SD with X Button
		// Exit with Y Button
		// ----------------------------------------------------------------------
		padReader.ReadLatest(&padStatus);

		if (txtWinSort == 0)
		{
			menuWnd.Ctrl(&padStatus);
		}

		// L Button Determination (minus)
		if (padStatus.trigger & BUTTON_L)
		{
			txtWinSort -= (txtWinSort > 0) ? 1 : -2;
		}

		// R Button Determination (plus)
		if (padStatus.trigger & BUTTON_R)
		{
			txtWinSort += (txtWinSort < 2) ? 1 : -2;
		}

		// Switch focus
		menuWnd.SetForcus(		txtWinSort == 0);
		resultWnd.SetForcus(	txtWinSort == 1);
		dbgTimerWnd.SetForcus(	txtWinSort == 2);

		// Output to SD with the X Button
		if (padStatus.trigger & BUTTON_X)
		{
			if (dbg_init == false)
			{
				dbg_init = dbgWriter.Initialize();
			}

			// Outputs only when initialization succeeds
			if (dbg_init == true)
			{
				dbgWriter.Output(rgbBuffer, proc_width, proc_height, &mgResult);
			}
		}

		// Quit with Y Button
		if (padStatus.trigger & BUTTON_Y)
		{
			break;
		}

		dbgTimerWnd.TimerEnd(6);

		// ----------------------------------------------------------------------
		// Swap
		// ----------------------------------------------------------------------
		dbgTimerWnd.TimerStart(7);
		graphics.SwapBuffer(NN_GX_DISPLAY1);
		dbgTimerWnd.TimerEnd(7);

		// ----------------------------------------------------------------------
		// wait vsync
		// ----------------------------------------------------------------------
		graphics.WaitVsync();
		dbgTimerWnd.TimerAllEnd();
	}

	///////////////////////////
	///////////////////////////
	///////////////////////////
	// Free all memory areas.
	///////////////////////////
	///////////////////////////
	///////////////////////////
    OS_FREE( rgbBuffer );

	if (extBuf != NULL)
	{
		OS_FREE(extBuf);
		extBuf = NULL;
	}

	if (mgCimg.pcImg != NULL)
	{
		OS_FREE(mgCimg.pcImg);
		mgCimg.pcImg = NULL;
	}

	graphics.FinalizeGraphics();
	camera.FinalizeCamera();

	// Destroy hid
	nn::hid::Finalize();

	FILE_IO_TERM();
	OS_MEM_TERM();

	//////////////////
	//////////////////
	//////////////////
	// Return.
	//////////////////
	//////////////////
	//////////////////
	PRINT_OUT("Program END\n");

	return ;
}

//	End of File ////////////////////////////////////////////////////////////////////////
